life = life or {}

-- initialization: "randomly" populates the buffer
local initCode = [[
extern float prob;
float rand(vec2 n)
{
	return 0.5 + 0.5 *
	fract(sin(dot(n.xy, vec2(12.9898, 78.233)))* 43758.5453);
}
vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
{
	float val = rand(pixel_coords);
	val = step(val, prob);
	return vec4(val,val,val,1.0);
}
]]

-- compute next generation (previous state is in texture)
local generation
local generationCode =  [[
extern float step_width;
extern float step_height;
vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
{
	float neighbours = 0.0;
	vec4 cell = Texel(texture, texture_coords);

	vec4 neighb = Texel(texture, texture_coords + vec2(-step_width,-step_height));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(0,-step_height));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(step_width,-step_height));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(-step_width,0));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(step_width,0));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(-step_width,step_height));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(0,step_height));
	neighbours += neighb.r;
	neighb = Texel(texture, texture_coords + vec2(+step_width,step_height));
	neighbours += neighb.r;
	float rval = step(cell.r,0.1) * step(3,neighbours) * step(neighbours,3);
	rval += step(0.1,cell.r) * step(2,neighbours) * step(neighbours,3);
	return vec4(rval,rval,rval,1.0);
}
]]

local fbsrc -- frame buffer 1
local fbdest -- frame buffer 2
local lg = love.graphics
local function initCanvas(canvas, p)
	local init = lg.newPixelEffect(initCode)
	print(init:getWarnings())
	init:send("prob",p)
	canvas:renderTo(function()
		lg.setPixelEffect(init)
		lg.rectangle('fill',0,0,life.width,life.height)
		lg.setPixelEffect()
	end)
end
initCanvas = function(canvas, p)
	local img = canvas:getImageData()
	img:mapPixel(function(x, y, r, g, b, a)
		if (math.random() < p) then
			return 255, 255, 255, 255
		else
			return 0, 0, 0, 255
		end
	end)
	canvas:renderTo(function ()
		lg.draw(lg.newImage(img),0,0)
	end)
end


function love.load()
	lg.setBackgroundColor(0,0,0)
	
	local modes = lg.getModes()
	table.sort(modes, function(a, b) return a.width * a.height < b.width * b.height end)
	local mode = modes[# modes]
	life.width = life.width or mode.width
	life.height = life.height or mode.height
	lg.setMode(life.width, life.height, true, false,0)
	
	fbsrc = fbsrc or lg.newCanvas(life.with, life.height)
	fbdest = fbdest or lg.newCanvas(life.with, life.height)
	generation = generation or lg.newPixelEffect(generationCode)
	print(generation:getWarnings())
	generation:send("step_width",1.0 / life.width)
	generation:send("step_height",1.0 / life.height)

	initCanvas(fbdest, life.prob or 0.10)
end

function love.update(dt)
	fbsrc, fbdest = fbdest, fbsrc
	fbdest:renderTo(function ()
		lg.setPixelEffect(generation)
		lg.draw(fbsrc)
		lg.setPixelEffect()
	end)
end

function love.draw()
	lg.draw(fbdest, 0, 0)
	lg.print((string.format("%.0f", 1.0 / love.timer.getDelta())), 5, 5, 0, 1)
end

function love.keypressed(key, unicode)
	if key == 'escape' then
		love.event.quit() -- love.event.push('q') -- quit the game
	end 
end
